import base64
import hashlib
import json
from typing import Optional
from urllib.request import urlopen

from packaging.version import parse as vp

STATIC_PATH = "drf_spectacular_sidecar/static/drf_spectacular_sidecar"
JSDELIVR_META_URL = "https://data.jsdelivr.com/v1/package/npm"
JSDELIVR_DATA_URL = "https://cdn.jsdelivr.net/npm"

FILES = {
    "redoc": [
        "bundles/redoc.standalone.js",
        "bundles/redoc.standalone.js.map",
        "bundles/redoc.standalone.js.LICENSE.txt",
    ],
    "swagger-ui-dist": [
        "swagger-ui-bundle.js",
        "swagger-ui-bundle.js.map",
        "swagger-ui-standalone-preset.js",
        "swagger-ui-standalone-preset.js.map",
        "oauth2-redirect.html",
        "swagger-ui.css",
        "swagger-ui.css.map",
        "favicon-32x32.png",
    ]
}

with open("distributions.json") as fh:
    _CURRENT_VERSIONS = json.load(fh)


def get_jsdelivr_tags(package, tag) -> str:
    url = f"{JSDELIVR_META_URL}/{package}"
    data = json.loads(urlopen(url).read())
    return data["tags"][tag]


def get_jsdelivr_package_content(package, version) -> dict:
    url = f"{JSDELIVR_META_URL}/{package}@{version}"
    return json.loads(urlopen(url).read())


def parse_jsdelivr_package_tree(node, path=""):
    path = (path + "/" if path else "") + node.get("name", "")
    if "default" in node or node["type"] == "directory":
        for i in node["files"]:
            yield from parse_jsdelivr_package_tree(i, path)
    elif node["type"] == "file":
        yield path, node["hash"]


def get_file_hash(file) -> str:
    with open(file, "rb") as fh:
        return base64.b64encode(hashlib.sha256(fh.read()).digest()).decode()


def validated_download(url, target, expected_hash) -> None:
    path = f"{STATIC_PATH}/{target}"
    print(f"download {url} to {target} [{expected_hash}]")
    with open(path, "wb") as fh:
        fh.write(urlopen(url).read())
    if expected_hash is not None:
        assert get_file_hash(path) == expected_hash


def update_dist(package, tag) -> Optional[str]:
    old_version = _CURRENT_VERSIONS[package]
    new_version = get_jsdelivr_tags(package, tag)
    print(f"'{package}' package tag '{tag}' has version '{new_version}'")
    if vp(new_version) <= vp(old_version):
        print(f"'{package}' package is up-to-date")
        return None
    package_hashes = dict(
        parse_jsdelivr_package_tree(get_jsdelivr_package_content(package, new_version))
    )
    for asset in FILES[package]:
        assert package_hashes[asset] is not None
        validated_download(
            url=f"{JSDELIVR_DATA_URL}/{package}@{new_version}/{asset}",
            target=f"{package}/{asset}",
            expected_hash=package_hashes[asset],
        )
    print(f"updated '{package}' from {old_version} to {new_version}")
    _CURRENT_VERSIONS[package] = new_version
    with open("distributions.json", 'w') as fh:
        json.dump(_CURRENT_VERSIONS, fh, indent=4)
    return new_version


def update_redoc() -> tuple[str, Optional[str]]:
    old_version = _CURRENT_VERSIONS["redoc"]
    return old_version, update_dist(package="redoc", tag="latest")


def update_swagger_ui() -> tuple[str, Optional[str]]:
    old_version = _CURRENT_VERSIONS["swagger-ui-dist"]
    new_version = update_dist(package="swagger-ui-dist", tag="latest")
    if new_version:
        # download license from GH as it is unfortunately not packaged in swagger-ui-dist
        validated_download(
            url=f"https://raw.githubusercontent.com/swagger-api/swagger-ui/v{new_version}/LICENSE",
            target="swagger-ui-dist/swagger-ui-bundle.js.LICENSE.txt",
            expected_hash=None
        )
    return old_version, new_version



import os
import re

from setuptools import setup

name = "drf-spectacular-sidecar"
package = "drf_spectacular_sidecar"
description = "Serve self-contained distribution builds of Swagger UI and Redoc with Django"
url = "https://github.com/tfranzel/drf-spectacular-sidecar"
author = "T. Franzel"
author_email = "tfranzel@gmail.com"
license = "BSD"

with open("README.rst") as readme:
    long_description = readme.read()

with open("requirements/base.txt") as fh:
    requirements = [r for r in fh.read().split("\n") if not r.startswith("#")]


def get_version(package):
    init_py = open(os.path.join(package, "__init__.py")).read()
    return re.search(
        "^__version__ = ['\"]([^'\"]+)['\"]", init_py, re.MULTILINE
    ).group(1)


def get_packages(package):
    return [
        dirpath
        for dirpath, dirnames, filenames in os.walk(package)
        if os.path.exists(os.path.join(dirpath, "__init__.py"))
    ]


setup(
    name=name,
    version=get_version(package),
    url=url,
    license=license,
    description=description,
    long_description=long_description,
    long_description_content_type="text/x-rst",
    author=author,
    author_email=author_email,
    packages=get_packages(package),
    include_package_data=True,
    python_requires=">=3.6",
    install_requires=requirements,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Documentation",
        "Topic :: Software Development :: Code Generators",
    ],
    project_urls={
        "Source": "https://github.com/tfranzel/drf-spectacular-sidecar",
        "Documentation": "https://drf-spectacular.readthedocs.io",
    },
)



import json
import os
import re
import shutil
from datetime import date

from packaging.version import parse as vp

from update_assets import update_redoc, update_swagger_ui

PACKAGE = 'drf_spectacular_sidecar'


def update_file(file, func):
    file_tmp = file + '.tmp'
    with open(file) as fh_in, open(file_tmp, "w") as fh_out:
        for line in fh_in.readlines():
            fh_out.write(func(line))
    shutil.move(file_tmp, file)


def run_or_fail(cmd):
    assert not os.system(cmd), f"failed: '{cmd}'"


def get_config(name):
    with open("config.json") as fh:
        return json.load(fh).get(name)


def update_config(name, value):
    with open("config.json", "r") as fh:
        config = json.load(fh)
    config[name] = value
    with open("config.json", "w") as fh:
        json.dump(config, fh, indent=4)


def main():
    old_redoc_version, new_redoc_version = update_redoc()
    old_swagger_ui_version, new_swagger_ui_version = update_swagger_ui()

    if (
        not new_redoc_version
        and not new_swagger_ui_version
        and not get_config('hotfix_required')
    ):
        print('no updates available')
        return

    update_config('hotfix_required', False)

    with open(f"{PACKAGE}/__init__.py") as fh:
        old_version = re.search(r"__version__ = '([\d.]+)'\n", fh.read()).group(1)

    today = date.today()
    new_version = f'{today.year}.{today.month}.{today.day}'
    assert vp(old_version) < vp(new_version), 'sidecar version must be newer'

    update_file(
        file=f"{PACKAGE}/__init__.py",
        func=lambda l: l.replace(old_version, new_version)
    )
    if new_redoc_version:
        update_file(
            file="README.rst",
            func=lambda l: l.replace(old_redoc_version, new_redoc_version)
        )
    if new_swagger_ui_version:
        update_file(
            file="README.rst",
            func=lambda l: l.replace(old_swagger_ui_version, new_swagger_ui_version),
        )

    if os.environ.get("CI"):
        run_or_fail("git config user.name github-actions")
        run_or_fail("git config user.email github-actions@github.com")

    # update repo
    run_or_fail("git add .")
    run_or_fail(f"git commit -m 'version bump {new_version}'")
    run_or_fail(f"git tag -a '{new_version}' -m 'version {new_version}'")
    run_or_fail("git push --follow-tags")
    # build
    run_or_fail("python setup.py sdist bdist_wheel")
    run_or_fail("twine check dist/*")
    run_or_fail("twine upload dist/*")
    # cleanup
    shutil.rmtree('dist')
    shutil.rmtree('build')
    shutil.rmtree(f'{PACKAGE}.egg-info')


if __name__ == '__main__':
    main()
